! Program to do 3D synthetic modeling using my GPU code

program model

 use sep
 implicit none

 integer			       :: v_nz, v_nx, v_ny, s_nz, s_nx, s_ny
 real				       :: v_oz, v_ox, v_oy, s_oz, s_ox, s_oy
 real				       :: v_dz, v_dx, v_dy, s_dz, s_dx, s_dy

 integer                               :: it, isx, isy, isz, dev_numin, wfld_j, nt, s_z, s_x, s_y
 real                                  :: dt, bc_a, bc_b

 real, dimension(:), allocatable       :: wavelet
 real, dimension(:,:,:), allocatable   :: vel, wavelet2, data


 call sep_init()

 ! Read 3D velocity model geometry
 call from_history("n1",v_nz)
 call from_history("o1",v_oz)
 call from_history("d1",v_dz)
 call from_history("n2",v_nx)
 call from_history("o2",v_ox)
 call from_history("d2",v_dx)
 call from_history("n3",v_ny)
 call from_history("o3",v_oy)
 call from_history("d3",v_dy)

 ! Read / construct shot geometry
 call from_param("s_nz",s_nz)
 call from_param("s_oz",s_oz)
 call from_param("s_dz",s_dz)
 call from_param("s_nx",s_nx)
 call from_param("s_ox",s_ox)
 call from_param("s_dx",s_dx)
 call from_param("s_ny",s_ny)
 call from_param("s_oy",s_oy)
 call from_param("s_dy",s_dy)

 ! Read wavelet and recording time
 call from_aux("wavelet","d1",dt)
 call from_aux("wavelet","n1",nt)

 ! Step stuff
 call from_param("wfld_j",wfld_j,3)

 ! Taper stuff
 call from_param("bc_a",bc_a)
 call from_param("bc_b",bc_b)   

 ! GPU stuff
 call from_param("dev_numin",dev_numin,0)

write(0,*) "data size of fast axis =", (nt-1)/wfld_j+1 

  call to_history("n1", v_nx)
  call to_history("o1", v_ox)
  call to_history("d1", v_dx)
  call to_history("n2", v_ny)
  call to_history("o2", v_oy)
  call to_history("d2", v_dy)
  call to_history("n3", (nt-1)/wfld_j+1)
  call to_history("o3", 0.0)
  call to_history("d3", dt*wfld_j)
  call to_history("n4", s_nx)
  call to_history("o4", s_ox*v_dx)
  call to_history("d4", s_dx*v_dx)
  call to_history("n5", s_ny)
  call to_history("o5", s_oy*v_dy)
  call to_history("d5", s_dy*v_dy)
  call to_history("label1", "x(k)")
  call to_history("label2", "y(km)")
  call to_history("label3", "t(s) ")
  call to_history("label4", "sx ")
  call to_history("label5", "sy ")


 
  call sep_close

 ! Allocation
  allocate( wavelet(nt))

  allocate( vel(v_nz, v_nx, v_ny))
  allocate( data((nt-1)/wfld_j+1,v_nx,v_ny) )

  data=0.

 ! Read in the velocity model
  call sreed("in", vel, 4*v_nz*v_nx*v_ny)
 ! Read in wavelet
  call sreed("wavelet", wavelet, 4*nt)

  call c_init_tex(nt, dt, v_nz, v_nx, v_ny, v_dz, v_dx, v_dy, bc_a, bc_b, wfld_j, dev_numin)

  vel=vel*vel*dt*dt
  write(0,*) "*********************** Propagate **************************"
  write(0,*) "*****************No shots will be  ",s_ny*s_nx*s_nz ,  "******************"
    do isy = 1,s_ny; s_y = (s_oy + (isy-1)*s_dy)
      do isx = 1,s_nx; s_x = s_ox + (isx-1)*s_dx
        do isz = 1,s_nz; s_z = (s_oz + (isz-1)*s_dz)

	  write(0,*) "Source positioned at",s_z,s_x,s_y

          call c_wave_prop_tex(s_z, s_x, s_y, wavelet, vel, data)

          call srite("out",data,4*v_nx*v_ny*((nt-1)/wfld_j+1))
       end do
      end do
    end do

	write(0,*) "*************** Through propagation kernel *****************"


end program model







