/* a tool to retreive parameters from command line arguments */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

/* return 0 if not found, 1 if found */
int scanKey(char *token, char *key, char **target){
	int keylen=strlen(key);
	int tokenLen=strlen(token);
	int j;
	for(j=0;(j<tokenLen && token[j]!='='); j++) ;
	if(j==tokenLen || j!=keylen) return 0; // This is if we miss the = sign
	char *temp=(char*) calloc(sizeof(char),j);
	strncpy(temp,token,j);
	if(strcmp(temp, key)!=0) {
		free(temp);
		return 0; // same len but mismatch
	}else{
		free(temp);
		*target=(char*)calloc(sizeof(char),strlen(token)-j);
		strcpy((char*)*target,token+j+1);
	}
	return 1;
}

/* optain a string from command line argument */
int cfrom_params(int argc, char *argv[], char *key, char **target, char *default_value){
	int found=0;
	for(int i=1; i<argc; i++)
		found+=scanKey(argv[i], key, target);
	if(found==0){
		*target=(char*)calloc(sizeof(char),500);
		strcpy((char*)*target, default_value);
	}
	return found;
}

/* optain an optional string from a header file */
int cfrom_hdrs(char *hfile, char *key, char **target, char *default_value){
	int found=0;
	FILE *fp=fopen(hfile,"r");
	if(fp==NULL){
		printf("Can't load the header file %s \n", hfile);
		exit(-1);
	}
	char *token=(char*)calloc(sizeof(char),200);
	while(!feof(fp)){
		fscanf(fp, "%s", token);
		found+=scanKey(token, key, target);
	}
	free(token);
	if(found==0){ 
		*target=(char*)calloc(sizeof(char),strlen(default_value));
		strcpy((char*) *target,default_value);
	}
	fclose(fp);
	return found;
}

/* */
int ifrom_params(int argc, char*argv[], char *key, int *target, int default_value){
	*target=default_value;
	int found=0;
	char *strval;
	for(int i=1; i<argc; i++)
		found+=scanKey(argv[i], key, &strval);
	if(found){
		*target=atoi(strval);
		free(strval);
	}
	return found;
}

/* optain an optional integer from a header file */
int ifrom_hdrs(char *hfile, char *key, int *target, int default_value){
	int found=0;
	FILE *fp=fopen(hfile,"r");
	if(fp==NULL){
		printf("Can't load the header file %s \n", hfile);
		exit(-1);
	}
	char *token=(char*)calloc(sizeof(char),200);
	char *strval;
	while(!feof(fp)){
		fscanf(fp, "%s", token);
		found+=scanKey(token, key, &strval);
	}
	free(token);
	if(found){ 
		target[0]=atoi(strval);
		free(strval);
	}else{
		*target=default_value;
	}
	fclose(fp);
	return found;
}

/*   */
int ffrom_params(int argc, char*argv[], char *key, float* target, float default_value){
	*target=default_value;
	int found=0;
	char *strval;
	for(int i=1; i<argc; i++)
		found+=scanKey(argv[i], key, &strval);
	if(found){
		target[0]=atof(strval);
		free(strval);
	}
	return found;
}

/* optain an optional string from a header file */
int ffrom_hdrs(char *hfile, char *key, float *target, float default_value){
	int found=0;
	FILE *fp=fopen(hfile,"r");
	if(fp==NULL){
		printf("Can't load the header file %s \n", hfile);
		exit(-1);
	}
	char *token=(char*)calloc(sizeof(char),200);
	char *strval;
	while(!feof(fp)){
		fscanf(fp, "%s", token);
		found+=scanKey(token, key, &strval);
	}
	free(token);
	if(found){ 
		*target=atof(strval);
		free(strval);
	}
	fclose(fp);
	return found;
}
